#include <Wire.h>

#define packetsize 30 //packetsize is set within the radio module, must match it here
struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  msrLspeed;  //measured wheel speeds
  int16_t  msrRspeed;
  uint16_t battVoltADC;
  byte operatingControlMode;
  int16_t  cmdLspeed;  //speed desired
  int16_t  cmdRspeed;
  int16_t  rmpLspeed;  //speed desired, smoothed to satisfy accel/decel
  int16_t  rmpRspeed;
  uint16_t accel;      
  uint16_t decel;
  int16_t  msrLpwr;    //power the pid applies to motors
  int16_t  msrRpwr;
  bool motorEN;        //state of switch that enables driving
  bool isParked;       //brakes on or off?
  uint8_t  temp1;   //pad this out to 30 bytes (to match the setup of the lora-promini radios)
  uint8_t  temp2;
  uint8_t  temp3;
  uint8_t  temp4;
  uint8_t  temp5;
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;
void setup() {
  Wire.begin();
  Serial.begin(115200);
  Serial.println("h");
}

void loop() {
  goRadio();
  debugFrame();
  delay(500);
}

void goRadio(){
  Wire.beginTransmission(8);
  Wire.write(frame.b, sizeof(frame.b));
  Wire.endTransmission(); //radio frame has been sent
  delay(5);
  byte rindex=255; //index for recived bytes
  bool rxvalidity=0;
  Wire.requestFrom(8, packetsize+1);
  while(Wire.available()){
    if(rindex==255){rxvalidity=Wire.read(); Serial.println(rxvalidity);}else{ //first read the status of the recived data
      if(rxvalidity==1){ //there is a valid frame waiting for us, so load it into the frame
        if(rindex>=packetsize-1){rindex=packetsize-1;} //overflow protection
        frame.b[rindex]=Wire.read();
      }else{
        Wire.read(); //not valid data recived. Throw it away.
      }
    }
    rindex++;
  }
}

void debugFrame(){ //prints the Frame content to serial port
  Serial.println();
  Serial.print(F("measured speeds: "));Serial.print(frame.s.msrLspeed);Serial.print('\t');Serial.println(frame.s.msrRspeed);
  Serial.print(F("batt volt adc  : "));Serial.println(frame.s.battVoltADC);
  Serial.print(F("operControlMode: "));Serial.println(frame.s.operatingControlMode);
  Serial.print(F("comman'd speeds: "));Serial.print(frame.s.cmdLspeed);Serial.print('\t');Serial.println(frame.s.cmdRspeed);
  Serial.print(F("ramp'd   speeds: "));Serial.print(frame.s.rmpLspeed);Serial.print('\t');Serial.println(frame.s.rmpRspeed);
  Serial.print(F("accel, decel   : "));Serial.print(frame.s.accel);Serial.print('\t');Serial.println(frame.s.decel);
  Serial.print(F("motor power    : "));Serial.print(frame.s.msrLpwr);Serial.print('\t');Serial.println(frame.s.msrLpwr);
  Serial.print(F("motor enable   : "));Serial.println(frame.s.motorEN);
  Serial.print(F("isParked       : "));Serial.println(frame.s.isParked);
}
