// rf95_client.pde - from the RadioHead demos
#define DIO0 3
#define DIO1 3
#define RESET 5
#define NSS 6
#include <util/crc16.h>

#include <Wire.h>

#include <SPI.h>
#include <RH_RF95.h>
RH_RF95 rf95(NSS, DIO0); //an arduino pro mini with bare RFM95
uint8_t buf[RH_RF95_MAX_MESSAGE_LEN]; // Should be a message for us now 
uint8_t len = sizeof(buf);

#define packetsize 30 //this is Wire max size
uint8_t rxd[packetsize]; //data packets recived 
uint8_t txd[packetsize]; //data packets to send
bool rxdvalid=0;

bool txdFlag=0; //when 1, do a request to the serer 

void dumpData(){
  Serial.println(F("buf:"));
  for(int i=0; i<packetsize; i++){
    Serial.print(i);
    Serial.print('\t');
    Serial.print(txd[i]);
    Serial.print('\t');
    Serial.print(rxd[i]);
    Serial.println();
  }
}

void setup(){
  pinMode(DIO1, INPUT_PULLUP); //not using this pin right now
  
  digitalWrite(RESET, LOW); //reset radio
  delay(200);
  pinMode(RESET, INPUT_PULLUP);
  
  Serial.begin(115200);
  if(!rf95.init()){
    Serial.println(F("init failed"));
  }else{Serial.println(F("init ok"));}

  rf95.setModemConfig(1); //Bw500Cr45Sf128
  rf95.setFrequency(915.0);
  rf95.setTxPower(23,false); //set RFM95 between 5 and 23

  Wire.begin(8);
  Wire.onRequest(requestEvent);
  Wire.onReceive(receiveEvent);
}

void requestEvent(){
  //copy rxd data, validity, and send it out
  buf[0]=rxdvalid;
  for(byte i=0; i<packetsize; i++){
    buf[i+1]=rxd[i]; 
  }
  Wire.write(buf,packetsize+1);
  rxdvalid=0; //now that data has been read out, assume invalid
}
void receiveEvent(int howMany) {
  Serial.println(howMany);
  byte iindex=0;
  while(Wire.available()){
    Serial.print(iindex);
    txd[iindex]=Wire.read();
    Serial.print('\t');
    Serial.println(txd[iindex]);
    iindex++;
    if(iindex>=packetsize-1){iindex=packetsize-1;}
  }
  txdFlag=1;
}

void loop(){
  if(txdFlag==1){
    txdFlag=0;
    transmit();
  }
}

void transmit(){ //sends txd buffer, then waits for reception - placed in rxd buffer
  Serial.println("send txd:");

  
  uint16_t txcrc=CRC16(txd, packetsize);
  buf[0]=txcrc>>8;
  buf[1]=txcrc;
  for(byte i=0;i<packetsize;i++){
    buf[i+2]=txd[i];
  }
  rf95.send(buf, packetsize+2);
  rf95.waitPacketSent();

  
  if(rf95.waitAvailableTimeout(100)){//Should be a reply message for us now   
    if(rf95.recv(buf, &len)){
      uint16_t rxcrc=buf[0]<<8; rxcrc+=buf[1]; //get the crc sent along with the data
      for(int i=0; i<packetsize; i++){//shift all the buf data up 2 places for crc calc
          buf[i]=buf[i+2];
      }
      if(CRC16(buf, packetsize)==rxcrc){ //calculate crc of rxd data, if match, then data ok
        for(int i=0; i<packetsize; i++){
          rxd[i]=buf[i];
        }
        Serial.println(F("got rxd"));
        rxdvalid=1;
      }else{Serial.println(F("bad data"));}
      
      
      //      Serial.print("RSSI: ");
      //      Serial.println(rf95.lastRssi(), DEC);
   
    }else{ Serial.println("recv failed"); }
  }else{ Serial.println("No reply, is rf95_server running?"); }
}

uint16_t CRC16(uint8_t *data, uint8_t lent){ //crc16 based on AVR _crc16_update
  uint16_t crc=0;
  for (int i=0;i<lent;i++){
    crc= _crc16_update (crc, data[i]); // update the crc value
  }
  return crc;
}
