#include <SPI.h>
#include "RF24.h"
RF24 radio(8,10); //use SPI + 8 and 9
byte addresses[][6] = {"radio","motor"};

struct cusend_t{  //structure that the control unit sends to the motor board
  //uint8_t  command; //command to motor controller
  int16_t  joysX;   //joystick x
  int16_t  joysY;   //joystick y
  //uint8_t  millissend; //millisecond of sent (extra source for error checking)
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union cusendU{
 cusend_t s;
 uint8_t b[sizeof(cusend_t)];
};
cusendU controls;

struct mcsend_t{  //structure that the control unit sends to the motor board
  //uint8_t  command; //command to motor controller
  int16_t  vbattadc;  //adc reading on battery
  int16_t  motorL; //motor L speed
  int16_t  motorR; //motor R speed 
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union mcsendU{
 mcsend_t s;
 uint8_t b[sizeof(mcsend_t)];
};
mcsendU mcresponse;

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);
unsigned long nextmillisDisp;

unsigned long nextmillis=1000;
#define smoothing 0.01
#define joystickZeroX 8.3
#define joystickZeroY 7.1
float joystickRawX;
float joystickRawY;

void setup(){
  Serial.begin(115200);
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.display();
  delay(2000);
  display.clearDisplay();
 
  radio.begin();
  radio.setPALevel(RF24_PA_LOW); //amplifier power level
  radio.openWritingPipe(addresses[1]);
  radio.openReadingPipe(1,addresses[0]);
  //radio.startListening();

  display.setTextSize(1);
  display.setTextColor(WHITE);
}

void loop(){
  joystickRawX = smoothing*(analogRead(A1)-joystickZeroX)+joystickRawX*(1-smoothing);
  joystickRawY = smoothing*(analogRead(A0)-joystickZeroY)+joystickRawY*(1-smoothing); 
  if(millis()>=nextmillis){
    nextmillis+=75;
    float joystickX = (float)map(10*joystickRawX, 7600, 2400, -2550, 2550); 
    float joystickY = (float)map(10*joystickRawY, 7600, 2400, -2550, 2550);
    joystickX=constrain(joystickX, -2550, 2550); joystickY=constrain(joystickY, -2550, 2550);
    if(sqrt(pow(joystickX,2)+pow(joystickY,2))<150){joystickX=0;joystickY=0;}
    //joystickX/=10; joystickY/=10;
    controls.s.joysX=joystickX; controls.s.joysY=joystickY;
    controls.s.checksum = CRC8(controls.b, sizeof(controls.b)-1);
    Serial.print(joystickX);Serial.print(',');Serial.println(joystickY);
    //printByteArray(controls.b, sizeof(controls.b));
    radio.write(&controls.s, sizeof(controls.s));

    int rxtimeout=1;
    radio.startListening();
    while(!radio.available()){
      if(millis()>=nextmillis-10){
          rxtimeout=0;
          break;
      }
    }
    if(rxtimeout){
      radio.read(&mcresponse.s, sizeof(mcresponse.s));
      byte cchecksum = CRC8(mcresponse.b, sizeof(mcresponse.b)-1); //calculate and
      if(cchecksum==mcresponse.s.checksum){                        //compare checksum
        printByteArray(mcresponse.b, sizeof(mcresponse.b));
        //Serial.println(mcresponse.s.vbattadc);
      }
    }
    radio.stopListening();
  }
  delay(1);
  if(millis()>=nextmillisDisp){
    nextmillisDisp+=1000;
    display.clearDisplay();
    display.setCursor(0,0);
    display.print(F("vbatt ="));display.println((float)mcresponse.s.vbattadc*.02783);
    display.print(F("mleft ="));display.println((float)mcresponse.s.motorL);
    display.print(F("mright="));display.println((float)mcresponse.s.motorR);
    display.display();
  }
}

void printByteArray(const byte *data, byte len){
  Serial.print(F("{\t"));
  while (len--){
    byte extract = *data++;
  Serial.print(extract);Serial.print(F(",\t"));
  }
  Serial.println(F("}"));
}

byte CRC8(const byte *data, byte len) {
  //CRC-8 - based on the CRC8 formulas by Dallas/Maxim
  //code released under the therms of the GNU GPL 3.0 license
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}
