/* ATMEGA328p - For testing
#define     hDisablePin 4  //h-bridge  controller disable pin
#define parkingBrakePin 13  //relay that releases brake and applys power to motor (high=active)
#define   rightEnableHI 5  
#define    leftEnableHI 6
#define        rightALI 3 //PWM
#define        rightBLI 11 //PWM
#define         leftALI 9 //PWM
#define         leftBLI 10 //PWM
*/

#define     hDisablePin 18  //h-bridge  controller disable pin
#define parkingBrakePin 19  //relay that releases brake and applys power to motor (high=active)
#define   rightEnableHI 20  
#define    leftEnableHI 21
#define        rightALI 12 //PWM
#define        rightBLI 13 //PWM
#define         leftALI 14 //PWM
#define         leftBLI 15 //PWM

#define FRposAccel 100   //fwd/reverse speed change limit
#define FRposDecel 75 
#define FRnegAccel 100
#define FRnegDecel 75

#define ROAccel 20   //rotation speed change limit
#define RODecel 10

#define fwdSpeedLimit 255
#define revSpeedLimit 100   //-100 in reality
#define rotSpeedLimit 50    //maximum allowable difference in motor speeds (both + or -)

#define outLimits 255    //+/- limit for outputs

#define onStopRelay       100    //millis between motion desired and relay turn on
#define onRelayMotion     400    //millis between relay turn on and motion enable
#define offMotionRelayStop 2000  //millis between motion enable and (relay turn off && motion enable)

unsigned long lastmillis; //for dt time calculations
unsigned long stageEpoch; //for turning on or off the main power relay

float FRsetpoint;  //setpoints and working values
float FRworking;
float ROsetpoint;
float ROworking;

float leftOut;  //outputs left and right to motor
float rightOut;

bool movst1; //motion desired or in action
bool movst2; //power relay on / off
bool movst3; //ready to move

bool motorsEnabled=0;

void setup(){
  pinMode(hDisablePin,     OUTPUT); digitalWrite(hDisablePin,     1);
  pinMode(parkingBrakePin, OUTPUT); digitalWrite(parkingBrakePin, 0);
  pinMode(leftEnableHI,    OUTPUT); digitalWrite(leftEnableHI,    0);
  pinMode(rightEnableHI,   OUTPUT); digitalWrite(rightEnableHI,   0);
  pinMode(leftALI,         OUTPUT); digitalWrite(leftALI,         0);
  pinMode(rightALI,        OUTPUT); digitalWrite(rightALI,        0);
  pinMode(leftBLI,         OUTPUT); digitalWrite(leftBLI,         0);
  pinMode(rightBLI,        OUTPUT); digitalWrite(rightBLI,        0);
  //TCCR2B &= ~0b00000111; //clear PWM timers clock select bits 
  //TCCR1B &= ~0b00000111;
  //TCCR2B |=  0b00000001; //set PWM timer prescaler (sets pwm frequency)
  //TCCR1B |=  0b00000001;
  Serial.begin(9600);
  delay(1000);
}

void loop() {
   update();
   motors((int)leftOut,(int)rightOut);
   Serial.print(leftOut);Serial.print('\t'); Serial.print(rightOut);Serial.print('\t');
   //Serial.print(FRsetpoint);Serial.print('\t'); Serial.print(FRworking);Serial.print('\t');
   //Serial.print(ROsetpoint);Serial.print('\t'); Serial.print(ROworking);Serial.print('\t');
   //Serial.print('\t');Serial.print(movst2);
   Serial.println();
   delay(50);
   serialControl();
}

void update(){
  unsigned long dt;
  unsigned long nowmillis=millis();
  if(nowmillis!=lastmillis){
    dt=nowmillis-lastmillis;
    lastmillis=nowmillis;
  }

  //Conditioning input signals
  if(FRsetpoint>= fwdSpeedLimit){FRsetpoint= fwdSpeedLimit;}
  if(FRsetpoint<=-revSpeedLimit){FRsetpoint=-revSpeedLimit;}
  if(ROsetpoint>= rotSpeedLimit){ROsetpoint= rotSpeedLimit;}
  if(ROsetpoint<=-rotSpeedLimit){ROsetpoint=-rotSpeedLimit;}

  if(movst1&&movst3){
    //FOWWORD AND REVERSE MOTION CONTROLLERS  
    if(FRworking>=0&&FRworking<FRsetpoint){ //accel in + direction
      FRworking+=(float)FRposAccel/1000*dt;
      if(FRworking>=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking>=0&&FRworking>FRsetpoint){ //decel in + direction
      FRworking-=(float)FRposDecel/1000*dt;
      if(FRworking<=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking<=0&&FRworking<FRsetpoint){ //decel in - direction
      FRworking+=(float)FRnegDecel/1000*dt;
      if(FRworking>=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking<=0&&FRworking>FRsetpoint){ //accel in - direction
      FRworking-=(float)FRnegAccel/1000*dt;
      if(FRworking<=FRsetpoint){FRworking=FRsetpoint;}
    }
   //ROTATION (TURNING) MOTON CONTROLLERS
  if(ROworking>=0&&ROworking<ROsetpoint){ //accel in + direction
      ROworking+=(float)ROAccel/1000*dt;
      if(ROworking>=ROsetpoint){ROworking=ROsetpoint;}
    }
    if(ROworking>=0&&ROworking>ROsetpoint){ //decel in + direction
      ROworking-=(float)RODecel/1000*dt;
      if(ROworking<=ROsetpoint){ROworking=ROsetpoint;}
    }
  
    if(ROworking<=0&&ROworking<ROsetpoint){ //decel in - direction
      ROworking+=(float)RODecel/1000*dt;
      if(ROworking>=ROsetpoint){ROworking=ROsetpoint;}
    }
    if(ROworking<=0&&ROworking>ROsetpoint){ //accel in - direction
      ROworking-=(float)ROAccel/1000*dt;
      if(ROworking<=ROsetpoint){ROworking=ROsetpoint;}
    }
  }

  if(FRworking+abs(ROworking)>= outLimits){FRworking= outLimits-abs(ROworking);}
  if(FRworking-abs(ROworking)<=-outLimits){FRworking=-outLimits+abs(ROworking);}
  
  leftOut =FRworking-ROworking;
  rightOut=FRworking+ROworking;

  if(leftOut >=outLimits){leftOut =outLimits;}  if(leftOut <=-outLimits){leftOut =-outLimits;}
  if(rightOut>=outLimits){rightOut=outLimits;}  if(rightOut<=-outLimits){rightOut=-outLimits;}


  if(ROworking==0&&FRworking==0&&ROsetpoint==0&&FRsetpoint==0&&movst1==1){movst1=0; stageEpoch=millis();}
  if((ROsetpoint!=0||FRsetpoint!=0)&&movst1==0){movst1=1; stageEpoch=millis();}

  if(movst1==1&&movst2==0&&millis()-stageEpoch>onStopRelay){movst2=1; enableMotors();}
  if(movst2==1&&movst3==0&&millis()-stageEpoch>(onStopRelay+onRelayMotion)){movst3=1;}
  
  if(movst1==0&&movst2==1&&millis()-stageEpoch>offMotionRelayStop){movst2=0;movst3=0; disableMotors();}
}

void enableMotors(){
  if(motorsEnabled==1){return;}  //don't run if motors are already enabled (might be under power!)
  digitalWrite(hDisablePin,   1); //set everything to inital off (disabled) values
  digitalWrite(leftEnableHI,  0); 
  digitalWrite(rightEnableHI, 0);
  digitalWrite(leftALI,       0);
  digitalWrite(rightALI,      0);
  digitalWrite(leftBLI,       0);
  digitalWrite(rightBLI,      0);
  
  digitalWrite(parkingBrakePin, 1); //turn on main power relay
  delay(100);
  
  digitalWrite(hDisablePin,   0); //enable hbridges
  digitalWrite(leftEnableHI,  1); //enable left motor
  digitalWrite(rightEnableHI, 1); //enable right motor

  lastmillis=millis();
  
  motorsEnabled=1;
}

void disableMotors(){
  digitalWrite(hDisablePin,   1);   //disable hbridges
  digitalWrite(parkingBrakePin, 0); //turn off power
  delay(100);
  
  digitalWrite(hDisablePin,   1); //set everything to inital off (disabled) values
  digitalWrite(leftEnableHI,  0); 
  digitalWrite(rightEnableHI, 0);
  digitalWrite(leftALI,       0);
  digitalWrite(rightALI,      0);
  digitalWrite(leftBLI,       0);
  digitalWrite(rightBLI,      0);
  
  motorsEnabled=0;
}

void motors(int leftpwr, int rightpwr){
  if(motorsEnabled!=1){return;}  //never run without enabling motor
  digitalWrite(hDisablePin,   1); //disable the h bridge while changing settings
  
  if( leftpwr>=255){leftpwr=255;} if( leftpwr<=-255){leftpwr=-255;}
  if(rightpwr>=255){leftpwr=255;} if(rightpwr<=-255){leftpwr=-255;}
   
  if(leftpwr <0){ analogWrite(leftALI ,abs(leftpwr ));  analogWrite(leftBLI ,0);             }
    if(leftpwr >0){ analogWrite(leftALI ,0);            analogWrite(leftBLI ,abs(leftpwr));  }
  if(rightpwr<0){ analogWrite(rightALI,abs(rightpwr));  analogWrite(rightBLI,0);             }
    if(rightpwr>0){ analogWrite(rightALI,0);            analogWrite(rightBLI,abs(rightpwr)); }
  
  if(leftpwr ==0){ analogWrite(leftALI ,0);             analogWrite(leftBLI ,0);             }
  if(rightpwr==0){ analogWrite(rightALI,0);             analogWrite(rightBLI,0);             }
  
  digitalWrite(hDisablePin,   0); //re-enable h bridge
}



void serialControl(){  //Serial command interface for debugging / not needed for full program
  /*format: In Serial monitor, enter ASNNN.NNNNN
    where A is a letter (letter command),
          S is minus sign (Optional)
          NNN.NNN is a decimal number
    A      will become char  'verb'
    SNNNN  will become float 'noun'
  */ 
  char verb;
  float noun;
  char sbuffer[32];
  byte slen = 0;
  if(Serial.available()){
  for(byte i=0; i<=32; i++){sbuffer[i]=0;}
  while(slen <= 0){ slen=Serial.readBytesUntil(13,sbuffer,32);}
  String sstr;
  for(byte i=0; i<=slen; i++){
    //Serial.print(i);Serial.print(':');Serial.print(sbuffer[i]);Serial.print(':');Serial.println(sbuffer[i], HEX);
    sstr+=sbuffer[i]; }
  //Serial.println(sstr);
  sstr.remove(0,1); //remove first char
  verb = sbuffer[0];
  noun = sstr.toFloat();
  //Serial.println(verb); 
  //Serial.println(noun,6);
  }
  Serial.flush();
  if(slen >= 1){  //Set following code for what is to be controlled
    switch(verb){
      case 'd':
       FRsetpoint = noun;
       break;
      case 't':
       ROsetpoint = noun;
       break;
   }
   Serial.print(F("V='"));Serial.print(verb);Serial.print(F("' N="));Serial.println(noun);
  }
}
