#define FRposAccel 100   //fwd/reverse speed change limit
#define FRposDecel 75 
#define FRnegAccel 100
#define FRnegDecel 75

#define ROAccel 20   //rotation speed change limit
#define RODecel 10

#define fwdSpeedLimit 255
#define revSpeedLimit 100   //-100 in reality
#define rotSpeedLimit 50    //maximum allowable difference in motor speeds (both + or -)

#define leftOutLimit 255    //+/- limit for outputs
#define rightOutLimit 255

#define onStopRelay       100    //millis between motion desired and relay turn on
#define onRelayMotion     400    //millis between relay turn on and motion enable
#define offMotionRelayStop 2000  //millis between motion enable and (relay turn off && motion enable)

#define relayPin  13 //main power relay
#define hEnPin     7 //h bridge enable

unsigned long lastmillis; //for dt time calculations
unsigned long stageEpoch; //for turning on or off the main power relay

float FRsetpoint;  //setpoints and working values
float FRworking;
float ROsetpoint;
float ROworking;

float leftOut;  //outputs left and right to motor
float rightOut;

bool movst1; //motion desired or in action
bool movst2; //power relay on / off
bool movst3; //ready to move

void setup(){
  pinMode(relayPin, OUTPUT);
  pinMode(hEnPin, OUTPUT);    digitalWrite(hEnPin, HIGH);
  Serial.begin(9600);
  lastmillis=millis();
  delay(10);
}

void loop() {
   update();
   Serial.print(leftOut);Serial.print('\t'); Serial.print(rightOut);Serial.print('\t');
   //Serial.print(FRsetpoint);Serial.print('\t'); Serial.print(FRworking);Serial.print('\t');
   //Serial.print(ROsetpoint);Serial.print('\t'); Serial.print(ROworking);Serial.print('\t');
   //Serial.print('\t');Serial.print(movst2);
   Serial.println();
   delay(50);
   serialControl();
}

void update(){
  unsigned long dt;
  unsigned long nowmillis=millis();
  if(nowmillis!=lastmillis){
    dt=nowmillis-lastmillis;
    lastmillis=nowmillis;
  }

  //Conditioning input signals
  if(FRsetpoint>= fwdSpeedLimit){FRsetpoint= fwdSpeedLimit;}
  if(FRsetpoint<=-revSpeedLimit){FRsetpoint=-revSpeedLimit;}
  if(ROsetpoint>= rotSpeedLimit){ROsetpoint= rotSpeedLimit;}
  if(ROsetpoint<=-rotSpeedLimit){ROsetpoint=-rotSpeedLimit;}

  if(movst1&&movst3){
    //FOWWORD AND REVERSE MOTION CONTROLLERS  
    if(FRworking>=0&&FRworking<FRsetpoint){ //accel in + direction
      FRworking+=(float)FRposAccel/1000*dt;
      if(FRworking>=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking>=0&&FRworking>FRsetpoint){ //decel in + direction
      FRworking-=(float)FRposDecel/1000*dt;
      if(FRworking<=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking<=0&&FRworking<FRsetpoint){ //decel in - direction
      FRworking+=(float)FRnegDecel/1000*dt;
      if(FRworking>=FRsetpoint){FRworking=FRsetpoint;}
    }
    if(FRworking<=0&&FRworking>FRsetpoint){ //accel in - direction
      FRworking-=(float)FRnegAccel/1000*dt;
      if(FRworking<=FRsetpoint){FRworking=FRsetpoint;}
    }
   //ROTATION (TURNING) MOTON CONTROLLERS
  if(ROworking>=0&&ROworking<ROsetpoint){ //accel in + direction
      ROworking+=(float)ROAccel/1000*dt;
      if(ROworking>=ROsetpoint){ROworking=ROsetpoint;}
    }
    if(ROworking>=0&&ROworking>ROsetpoint){ //decel in + direction
      ROworking-=(float)RODecel/1000*dt;
      if(ROworking<=ROsetpoint){ROworking=ROsetpoint;}
    }
  
    if(ROworking<=0&&ROworking<ROsetpoint){ //decel in - direction
      ROworking+=(float)RODecel/1000*dt;
      if(ROworking>=ROsetpoint){ROworking=ROsetpoint;}
    }
    if(ROworking<=0&&ROworking>ROsetpoint){ //accel in - direction
      ROworking-=(float)ROAccel/1000*dt;
      if(ROworking<=ROsetpoint){ROworking=ROsetpoint;}
    }
  }

  leftOut =FRworking-ROworking;
  rightOut=FRworking+ROworking;

  if(leftOut>= leftOutLimit  ){leftOut=leftOutLimit;}   if(leftOut<=-leftOutLimit  ){leftOut=leftOutLimit;}
  if(rightOut>= rightOutLimit){rightOut=rightOutLimit;} if(rightOut<=-rightOutLimit){rightOut=rightOutLimit;}

  if(ROworking==0&&FRworking==0&&ROsetpoint==0&&FRsetpoint==0&&movst1==1){movst1=0; stageEpoch=millis();}
  if((ROsetpoint!=0||FRsetpoint!=0)&&movst1==0){movst1=1; stageEpoch=millis();}

  if(movst1==1&&movst2==0&&millis()-stageEpoch>onStopRelay){movst2=1; digitalWrite(relayPin,1);}
  if(movst2==1&&movst3==0&&millis()-stageEpoch>(onStopRelay+onRelayMotion)){movst3=1;}
  
  if(movst1==0&&movst2==1&&millis()-stageEpoch>offMotionRelayStop){movst2=0;movst3=0; digitalWrite(relayPin,0);}
}

void serialControl(){  //Serial command interface for debugging / not needed for full program
  /*format: In Serial monitor, enter ASNNN.NNNNN
    where A is a letter (letter command),
          S is minus sign (Optional)
          NNN.NNN is a decimal number
    A      will become char  'verb'
    SNNNN  will become float 'noun'
  */ 
  char verb;
  float noun;
  char sbuffer[32];
  byte slen = 0;
  if(Serial.available()){
  for(byte i=0; i<=32; i++){sbuffer[i]=0;}
  while(slen <= 0){ slen=Serial.readBytesUntil(13,sbuffer,32);}
  String sstr;
  for(byte i=0; i<=slen; i++){
    //Serial.print(i);Serial.print(':');Serial.print(sbuffer[i]);Serial.print(':');Serial.println(sbuffer[i], HEX);
    sstr+=sbuffer[i]; }
  //Serial.println(sstr);
  sstr.remove(0,1); //remove first char
  verb = sbuffer[0];
  noun = sstr.toFloat();
  //Serial.println(verb); 
  //Serial.println(noun,6);
  }
  Serial.flush();
  if(slen >= 1){  //Set following code for what is to be controlled
    switch(verb){
      case 'd':
       FRsetpoint = noun;
       break;
      case 't':
       ROsetpoint = noun;
       break;
   }
   Serial.print(F("V='"));Serial.print(verb);Serial.print(F("' N="));Serial.print(noun);
  }
}
