#include <SPI.h>
#include "RF24.h"
RF24 radio(10, 9); //use SPI + 8 and 9
byte addresses[][6] = {"radio", "motor"};

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x3F, 16, 2);

struct cusend_t { //structure that the control unit sends to the motor board
  uint8_t  command; //command to motor controller
  int16_t  joysX;   //joystick x
  int16_t  joysY;   //joystick y
  uint16_t  accel;
  uint16_t  decel;
  uint16_t  maxspeed;
  uint16_t  roaccel;
  uint16_t  rodecel;
  uint16_t  romaxspeed;
  //uint8_t  millissend; //millisecond of sent (extra source for error checking)
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union cusendU {
  cusend_t s;
  uint8_t b[sizeof(cusend_t)];
};
cusendU controls;

struct mcsend_t { //structure that the control unit sends to the motor board
  //uint8_t  response;
  int16_t  vbattadc;  //adc reading on battery
  int16_t  motorL; //motor L speed
  int16_t  motorR; //motor R speed
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union mcsendU {
  mcsend_t s;
  uint8_t b[sizeof(mcsend_t)];
};
mcsendU mcresponse;

unsigned long nextmillis = 1000;
int lcdupdate = 0;
#define smoothing 0.01
#define joystickZeroX 8.3
#define joystickZeroY 7.1
float joystickRawX;
float joystickRawY;

void setup() {
  Serial.begin(115200);
  pinMode(A6, INPUT); pinMode(A7, INPUT);
  pinMode(4, INPUT_PULLUP); pinMode(7, INPUT_PULLUP);
  lcd.init(); lcd.backlight();

  radio.begin();
  radio.setPALevel(RF24_PA_LOW); //amplifier power level
  radio.openWritingPipe(addresses[1]);
  radio.openReadingPipe(1, addresses[0]);
}

void loop() {
  joystickRawX = smoothing * (analogRead(A6) - joystickZeroX) + joystickRawX * (1 - smoothing);
  joystickRawY = smoothing * (analogRead(A7) - joystickZeroY) + joystickRawY * (1 - smoothing);
  if (millis() >= nextmillis) {
    nextmillis += 100;
    float joystickX = (float)map(10 * joystickRawX, 7600, 2400, -2550, 2550);
    float joystickY = (float)map(10 * joystickRawY, 7600, 2400, -2550, 2550);
    joystickX = constrain(joystickX, -2550, 2550);
    joystickY = constrain(joystickY, -2550, 2550);
    if (sqrt(pow(joystickX, 2) + pow(joystickY, 2)) < 230) {
      joystickX = 0;
      joystickY = 0;
    }

    if (digitalRead(7)) {
      joystickX = 0;
      joystickY = 0;
    }
    if (!digitalRead(4)) {
      joystickX = -joystickX;
      joystickY = -joystickY;
    } else {
      joystickX = -joystickX;
      joystickY = joystickY;
    }

    controls.s.joysX = joystickX;
    controls.s.joysY = joystickY;

    controls.s.maxspeed = analogRead(A1) / 3;
    controls.s.accel = analogRead(A2);
    controls.s.decel = analogRead(A0);

    controls.s.romaxspeed = 100;
    controls.s.roaccel = 200;
    controls.s.rodecel = 200;

    controls.s.checksum = CRC8(controls.b, sizeof(controls.b) - 1);
    
    printByteArray(controls.b, sizeof(controls.b));
    
    /*
    Serial.print(joystickX); Serial.print('\t'); Serial.print(joystickY);
    Serial.print('\t'); Serial.print(mcresponse.s.motorL);
    Serial.print('\t'); Serial.print(mcresponse.s.motorR);
    Serial.print('\t'); Serial.print(controls.s.maxspeed);
    Serial.print('\t'); Serial.print(controls.s.accel);
    Serial.print('\t'); Serial.print(controls.s.decel);
    Serial.println();
    */

    radio.write(&controls.s, sizeof(controls.s));
    int rxtimeout = 1;
    radio.startListening();
    while (!radio.available()) {
      if (millis() >= nextmillis - 10) {
        rxtimeout = 0;
        break;
      }
    }
    if (rxtimeout) {
      radio.read(&mcresponse.s, sizeof(mcresponse.s));
      byte cchecksum = CRC8(mcresponse.b, sizeof(mcresponse.b) - 1); //calculate and
      if (cchecksum == mcresponse.s.checksum) {                    //compare checksum
        //printByteArray(mcresponse.b, sizeof(mcresponse.b));
      }
    }
    radio.stopListening();

    if (lcdupdate == 0) {
      lcd.clear();
      lcd.print(mcresponse.s.vbattadc * .02783, 2);
      lcd.setCursor(0, 1);
      lcd.print(mcresponse.s.motorL);
      lcd.setCursor(7, 1);
      lcd.print(mcresponse.s.motorR);
    }
    lcdupdate++;
    if (lcdupdate >= 10) {
      lcdupdate = 0;
    }

  }
}

void printByteArray(const byte *data, byte len) {
  Serial.print(F("{\t"));
  while (len--) {
    byte extract = *data++;
    Serial.print(extract); Serial.print(F(",\t"));
  }
  Serial.println(F("}"));
}

byte CRC8(const byte *data, byte len) {
  //CRC-8 - based on the CRC8 formulas by Dallas/Maxim
  //code released under the therms of the GNU GPL 3.0 license
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}
