#include <SPI.h>
#include <RH_NRF24.h>
RH_NRF24 nrf24;
#include <Arduino.h>
#include <U8g2lib.h>
#include <Wire.h>
U8G2_SSD1306_128X64_NONAME_1_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);

struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  rxdLspeed;
  int16_t  rxdRspeed;
  uint16_t battVolt;
  uint8_t bitStatus;
  int16_t  cmdLspeed;
  int16_t  cmdRspeed;
  int16_t  cmdRotate;
  int16_t  cmdFSpeed; 
  uint16_t  fwdMaxSpeed;
  uint16_t  revMaxSpeed;
  uint16_t  fwdAccel;
  uint16_t  rwdDecel;
  uint16_t  revAccel;
  uint16_t  revDecel;
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;
byte CRC8(const byte *data, byte len) {
  //CRC-8 - based on the CRC8 formulas by Dallas/Maxim
  //code released under the therms of the GNU GPL 3.0 license
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}
void calcCRC8(){
  frame.s.checksum=0;
  frame.s.checksum = CRC8(frame.b, sizeof(frame.b)-1);
}
bool checkCRC8(){
  uint8_t existingChecksum=frame.s.checksum;
  calcCRC8();
  if(existingChecksum==frame.s.checksum){
    return 1;
  }else{
    return 0;
  }
}

unsigned long startLoopt=0;

#define jsxpin A6
#define jsypin A7
int jsx = 0;
int jsy = 0;
float jsxF = 0;
float jsyF = 0;

#define loopTime 75

void setup(){
  Serial.begin(9600);
  if(!nrf24.init())Serial.println("init failed"); // Defaults after init are 2.402 GHz (channel 2), 2Mbps, 0dBm
  if(!nrf24.setChannel(1))Serial.println("setChannel failed");
  if(!nrf24.setRF(RH_NRF24::DataRate2Mbps, RH_NRF24::TransmitPower0dBm))Serial.println("setRF failed");  
  u8g2.begin();  
}

void loop(){
  startLoopt=millis();
  frame.s.battVolt=millis();
  calcCRC8();
  //nrf24.send(frame.b, sizeof(frame.b));
  //nrf24.waitPacketSent();
  uint8_t buf[RH_NRF24_MAX_MESSAGE_LEN];
  uint8_t len = sizeof(buf);


  
  u8g2.firstPage();  
  do{
    draw();
  }while( u8g2.nextPage() );

  jsx = map(analogRead(jsxpin),0,1023,-1000,1000);
  jsy = map(analogRead(jsypin),0,1023,-1000,1000);
  jsxF = 0.6*jsxF + 0.4*jsx;
  jsyF = 0.6*jsyF + 0.4*jsy;
  
  while(millis()-startLoopt<loopTime){
    //Serial.println(millis()-startLoopt);
    yield();
  }
}
