#include <Arduino.h>
#include <U8x8lib.h>
#include <Wire.h>
U8X8_SSD1306_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE);    

byte encoderFlag=0; //1 for no change, 0 for ccw, 2 for cw
byte encoderFlag2=0; //1 for no change, 0 for ccw, 2 for cw, non-selfresetting
unsigned long encoderFlagTime=0;
#define encoderFlagTimeout 5
unsigned long joystickTime=0;
#define joystickTimeout 5

bool blackButtonFlag=0; //0 for not pressed, 1 for pressed
bool whiteButtonFlag=0;
bool encoderButtonFlag=0;
bool centerSwitch=0;
bool rightSwitch=0;
int rightPot=0;
int centerPot=0;
int leftPot=0;
float joystickXF=0;
float joystickYF=0;
int joystickX=0;
int joystickY=0;
bool joystickCenter=0;

float mrotate=0;
float mfwdrev=0;
float motorspeedLeft=0;
float motorspeedRght=0;

int steps;
unsigned long screenTime=0;
#define screenTimeout 50

void setup() {
   pinMode(A6, INPUT); pinMode(A7, INPUT); //joystick
   pinMode(A0, INPUT); pinMode(A1, INPUT); pinMode(A2, INPUT); //knobs
   pinMode(4, INPUT_PULLUP); pinMode(7, INPUT_PULLUP); //switches
   pinMode(3, INPUT_PULLUP); pinMode(2, INPUT_PULLUP); //encoder rotor
   pinMode(8, INPUT_PULLUP); pinMode(5, INPUT_PULLUP); pinMode(6, INPUT_PULLUP); //buttons
   u8x8.begin();
   u8x8.setFont(u8x8_font_pxplusibmcga_f);
   u8x8.clear();
   intInit();
   Serial.begin(9600);
}

void loop(){
  screenUpdate();
  uiUpdate();
}

void screenUpdate(){
  if(millis()-screenTime>screenTimeout){
    screenTime=millis();
    //u8x8.clear();
    u8x8.setCursor(0,2); uprintp(leftPot,4);
    u8x8.setCursor(5,2); uprintp(centerPot,4);
    u8x8.setCursor(10,2); uprintp(rightPot,4);
    
    u8x8.setCursor(0,1); u8x8.print(centerSwitch);
    u8x8.setCursor(1,1); u8x8.print(rightSwitch);
    u8x8.setCursor(3,1); u8x8.print(blackButtonFlag);
    u8x8.setCursor(4,1); u8x8.print(whiteButtonFlag);
    u8x8.setCursor(5,1); u8x8.print(encoderButtonFlag);
    u8x8.setCursor(7,1); u8x8.print(encoderFlag2);

    u8x8.setCursor(0,0); uprintp(joystickX, -3);
    u8x8.setCursor(5,0); uprintp(joystickY, -3);
    u8x8.setCursor(10,0); u8x8.print(joystickCenter);

    if(joystickCenter==1){
      mrotate=(float)leftPot*(float)joystickX/102.3;
      mfwdrev=(float)centerPot*(float)joystickY/102.3;
    }else{
      mrotate=0; mfwdrev=0;
    }
    u8x8.setCursor(0,3); uprintp(mrotate, -4);
    u8x8.setCursor(6,3); uprintp(mfwdrev, -4);

    if(abs(mfwdrev)+abs(mrotate)>=1000){
      mfwdrev=(mfwdrev/abs(mfwdrev))*(1000-abs(mrotate));
    }

    if(centerSwitch==0){
      motorspeedLeft=(mrotate+mfwdrev);
      motorspeedRght=(-mrotate+mfwdrev);
    }else{
      motorspeedLeft=-(mrotate+mfwdrev);
      motorspeedRght=-(-mrotate+mfwdrev);
    }

    

    u8x8.setCursor(0,4); uprintp(motorspeedLeft, -4);
    u8x8.setCursor(6,4); uprintp(motorspeedRght, -4);

    
    blackButtonFlag=0;
    whiteButtonFlag=0;
    encoderButtonFlag=0;
    encoderFlag2=1;
  }
}


char signs(int j){
  if(j>0){return 1;}
  return -1;
}

//************************Buttons/encoder interrupt sections****************************
void intInit(){ //interrupts for buttons and encoder
  cli();              //set up encoder interrupts on A1-A4
  PCICR |= 0b00000101;
  PCMSK0 |= 0b00000001;
  PCMSK2 |= 0b01101100;
  sei();
}

ISR(PCINT0_vect){  //encoder button interrupt
  if((PINB & 0b00000001)==0){encoderButtonFlag=1;}
}

ISR(PCINT2_vect){  //other button/encoder interrupt
  if((PIND & 0b01000000)==0){blackButtonFlag=1;}
  if((PIND & 0b00100000)==0){whiteButtonFlag=1;}
  if((PIND & 0b00000100)>>2==0&&(PIND & 0b00001000)>>3==1&&encoderFlag==255){encoderFlag=0;}
  if((PIND & 0b00000100)>>2==1&&(PIND & 0b00001000)>>3==0&&encoderFlag==255){encoderFlag=2;}
}

void uiUpdate(){
  if((PIND & 0b00000100)>>2==1&&(PIND & 0b00001000)>>3==1&&encoderFlag==254&&millis()-encoderFlagTime>encoderFlagTimeout){encoderFlag=255;}
  if(encoderFlag<254){
    encoderFlagTime=millis();
    steps+=encoderFlag;
    steps--;
    encoderFlag2=encoderFlag;
    encoderFlag=254; //disable further encoder readings until step complete
  }
  leftPot=analogRead(A2);
  centerPot=analogRead(A1);
  rightPot=analogRead(A0);
  centerSwitch=!digitalRead(4);
  rightSwitch=!digitalRead(7);
  if(millis()-joystickTimeout>joystickTime){
    joystickTime=millis();
    joystickXF=0.2*map(analogRead(A6),226,786,1000,-1000)+0.8*joystickXF;
    joystickYF=0.2*map(analogRead(A7),226,786,1000,-1000)+0.8*joystickYF;
    joystickX=constrain(joystickXF/10,-100,100);
    joystickY=constrain(joystickYF/10,-100,100);
    joystickCenter=(sqrt(joystickX*joystickX+joystickY*joystickY)>3);
  }
}

//**************************screen functions********************************************
void uprintp(int x, char z){ //print number with padding
  if(z<0){
    if(x<0){u8x8.print('-');}else{u8x8.print('+');}
  }
  z=abs(z);
  x=abs(x);
  for(byte n=1; n<z; n++){
    if(x<pow(10,n)){u8x8.print(0);}
  }
  u8x8.print(x);
}
