#include <Arduino.h>
#include <U8x8lib.h>
#include <Wire.h>
U8X8_SSD1306_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE);  

#define estopSWPin 2
#define controlmodeSWPin 6
#define drivedirSWPin 5
#define whiteBTPin 3
#define blackBTPin 4
#define temp1SWPin 9
#define temp2SWPin 8
#define temp3SWPin 7
#define joyXPin A6
#define joyYPin A7
#define speedKPin A0
#define accelKPin A1
#define decelKPin A2
#define ledPin 13


bool blackButtonFlag=0; //0 for not pressed, 1 for pressed
bool whiteButtonFlag=0;
bool encoderButtonFlag=0;
bool centerSwitch=0;
bool rightSwitch=0;
int rightPot=0;
int centerPot=0;
int leftPot=0;
float joystickXF=0;
float joystickYF=0;
int joystickX=0;
int joystickY=0;
bool joystickCenter=0;

float scalerotate=0;
float scalefwdrev=0;
float mrotate=0;
float mfwdrev=0;
float motorspeedLeft=0;
float motorspeedRght=0;
float accel=0;
float decel=0;

bool radioMotorEN=0;

int steps;
unsigned long screenTime=0;
#define screenTimeout 10

#define packetsize 30 //packetsize is set within the radio module, must match it here
struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  msrLspeed;  //measured wheel speeds
  int16_t  msrRspeed;
  uint16_t battVoltADC;
  byte operatingControlMode;
  int16_t  cmdLspeed;  //speed desired
  int16_t  cmdRspeed;
  int16_t  rmpLspeed;  //speed desired, smoothed to satisfy accel/decel
  int16_t  rmpRspeed;
  uint16_t accel;      
  uint16_t decel;
  int16_t  msrLpwr;    //power the pid applies to motors
  int16_t  msrRpwr;
  bool motorEN;        //state of switch that enables driving
  bool isParked;       //brakes on or off?
  uint8_t  temp1;   //pad this out to 30 bytes (to match the setup of the lora-promini radios)
  uint8_t  temp2;
  uint8_t  temp3;
  uint8_t  temp4;
  uint8_t  temp5;
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;

#define radioTimeout 500
unsigned long radioTime=0;

void setup() {
   pinMode(joyXPin, INPUT);
   pinMode(joyYPin, INPUT);
   pinMode(speedKPin, INPUT);
   pinMode(accelKPin, INPUT);
   pinMode(decelKPin, INPUT);
   pinMode(estopSWPin, INPUT_PULLUP);
   pinMode(controlmodeSWPin, INPUT_PULLUP);
   pinMode(drivedirSWPin, INPUT_PULLUP);
   pinMode(whiteBTPin, INPUT_PULLUP);
   pinMode(blackBTPin, INPUT_PULLUP);
   pinMode(temp1SWPin, INPUT_PULLUP);
   pinMode(temp2SWPin, INPUT_PULLUP);
   pinMode(temp3SWPin, INPUT_PULLUP);
   pinMode(ledPin, OUTPUT);
   u8x8.begin();
   u8x8.setFont(u8x8_font_pxplusibmcga_f);
   u8x8.clear();
   Serial.begin(115200);
   frame.s.operatingControlMode=1;
}

void goRadio(){
  Wire.beginTransmission(8);
  Wire.write(frame.b, sizeof(frame.b));
  Wire.endTransmission(); //radio frame has been sent
  delay(44); //wait for transmit + reception
  byte rindex=255; //index for recived bytes
  bool rxvalidity=0;
  Wire.requestFrom(8, packetsize+1);
  while(Wire.available()){
    if(rindex==255){rxvalidity=Wire.read(); if(rxvalidity==1){radioTime=millis();}}else{ //first read the status of the recived data
      if(rxvalidity==1){ //there is a valid frame waiting for us, so load it into the frame
        if(rindex>=packetsize-1){rindex=packetsize-1;} //overflow protection
        frame.b[rindex]=Wire.read();
      }else{
        Wire.read(); //not valid data recived. Throw it away.
      }
    }
    rindex++;
  }
}

void debugFrame(){ //prints the Frame content to serial port
  Serial.println();
  Serial.print(F("measured speeds: "));Serial.print(frame.s.msrLspeed);Serial.print('\t');Serial.println(frame.s.msrRspeed);
  Serial.print(F("batt volt adc  : "));Serial.println(frame.s.battVoltADC);
  Serial.print(F("operControlMode: "));Serial.println(frame.s.operatingControlMode);
  Serial.print(F("comman'd speeds: "));Serial.print(frame.s.cmdLspeed);Serial.print('\t');Serial.println(frame.s.cmdRspeed);
  Serial.print(F("ramp'd   speeds: "));Serial.print(frame.s.rmpLspeed);Serial.print('\t');Serial.println(frame.s.rmpRspeed);
  Serial.print(F("accel, decel   : "));Serial.print(frame.s.accel);Serial.print('\t');Serial.println(frame.s.decel);
  Serial.print(F("motor power    : "));Serial.print(frame.s.msrLpwr);Serial.print('\t');Serial.println(frame.s.msrLpwr);
  Serial.print(F("motor enable   : "));Serial.println(frame.s.motorEN);
  Serial.print(F("isParked       : "));Serial.println(frame.s.isParked);
}

void loop(){
  screenUpdate();
  uiUpdate();
}

void screenUpdate(){
  if(millis()-screenTime>screenTimeout){
    debugFrame();
    screenTime=millis();
    //u8x8.clear();

    u8x8.setCursor(13,0); if(millis()-radioTime>=radioTimeout){digitalWrite(ledPin, LOW);}else{digitalWrite(ledPin, HIGH);}
    
    u8x8.setCursor(0,0); uprintp(joystickX, -3);
    u8x8.setCursor(5,0); uprintp(joystickY, -3);
    u8x8.setCursor(10,0); u8x8.print(joystickCenter);

    scalefwdrev=(float)analogRead(speedKPin)/25;
    scalerotate=scalefwdrev/4;
    accel=analogRead(accelKPin)*4;
    decel=analogRead(decelKPin)*4;
    
    u8x8.setCursor(1,3); uprintp(scalefwdrev*100, 4);
    u8x8.setCursor(7,3); uprintp(accel, 4);
    u8x8.setCursor(12,3); uprintp(decel, 4);
    
    if(joystickCenter==0){ scalerotate=0; scalefwdrev=0; }
    
    mrotate=scalerotate*joystickX;
    mfwdrev=scalefwdrev*joystickY;
    if(abs(mfwdrev)+abs(mrotate)>=100*scalefwdrev){
      mfwdrev=(mfwdrev/abs(mfwdrev))*(100*scalefwdrev-abs(mrotate));
    }
    u8x8.setCursor(13,1);
    if(digitalRead(drivedirSWPin)==1){
      u8x8.print(F("F"));
      motorspeedLeft=(mrotate+mfwdrev);
      motorspeedRght=(-mrotate+mfwdrev);
    }else{
      u8x8.print(F("R"));
      motorspeedLeft=-(-mrotate+mfwdrev);
      motorspeedRght=-(mrotate+mfwdrev);
    }

    u8x8.setCursor(0,4); uprintp(motorspeedLeft, -4);
    u8x8.setCursor(6,4); uprintp(motorspeedRght, -4);

    frame.s.cmdLspeed=motorspeedLeft;
    frame.s.cmdRspeed=motorspeedRght;
    frame.s.accel=accel;
    frame.s.decel=decel;
    frame.s.motorEN=radioMotorEN;
    if(digitalRead(controlmodeSWPin)){frame.s.operatingControlMode=2;}else{frame.s.operatingControlMode=1;}

    u8x8.setCursor(7,1); if(frame.s.operatingControlMode==2){u8x8.print(F("FF"));}else{u8x8.print(F("FB"));}
    u8x8.setCursor(10,1); if(radioMotorEN==1){u8x8.print(F("EN"));}else{u8x8.print(F("DS"));}
    
    goRadio();

    u8x8.setCursor(0,1); u8x8.print(F("BV")); uprintp(frame.s.battVoltADC*02.783,4);
    u8x8.setCursor(0,5); uprintp(frame.s.rmpLspeed,-4);
    u8x8.setCursor(6,5); uprintp(frame.s.rmpRspeed,-4);
    u8x8.setCursor(0,6); uprintp(frame.s.msrLspeed,-4);
    u8x8.setCursor(6,6); uprintp(frame.s.msrRspeed,-4);
    u8x8.setCursor(0,7); uprintp(frame.s.msrLpwr,-3);
    u8x8.setCursor(6,7); uprintp(frame.s.msrRpwr,-3);
    u8x8.setCursor(12,7); if(frame.s.isParked){u8x8.print(F("PK"));}else{u8x8.print(F("MV"));}

    if(digitalRead(estopSWPin)==1){
      radioMotorEN=0;
    }else{
      radioMotorEN=1;
    }
  }
}

void uiUpdate(){
    joystickXF=map(analogRead(A6),226,786,1000,-1000);
    joystickYF=map(analogRead(A7),226,786,1000,-1000);
    joystickX=constrain(joystickXF/10,-100,100);
    joystickY=constrain(joystickYF/10,-100,100);
    joystickCenter=(sqrt(joystickX*joystickX+joystickY*joystickY)>3);
}

//**************************screen functions********************************************
void uprintp(int x, char z){ //print number with padding
  if(z<0){
    if(x<0){u8x8.print('-');}else{u8x8.print('+');}
  }
  z=abs(z);
  x=abs(x);
  for(byte n=1; n<z; n++){
    if(x<pow(10,n)){u8x8.print(0);}
  }
  u8x8.print(x);
}
