byte CRC8(const byte *data, byte len) {
  //CRC-8 - based on the CRC8 formulas by Dallas/Maxim
  //code released under the therms of the GNU GPL 3.0 license
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}

void calcCRC8(){
  frame.s.checksum=0;
  frame.s.checksum = CRC8(frame.b, sizeof(frame.b)-1);
}
bool checkCRC8(){
  uint8_t existingChecksum=frame.s.checksum;
  calcCRC8();
  if(existingChecksum==frame.s.checksum){
    return 1;
  }else{
    return 0;
  }
}

void checkRadio(){
  if(nrf24.available()){
    uint8_t buf[RH_NRF24_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);
    if(nrf24.recv(buf, &len)){
      //NRF24::printBuffer("request: ", buf, len);
      for(byte e=0;e<sizeof(frame.b);e++){
        frame.b[e]=buf[e];
      }
      if(checkCRC8()){
        //valid response received
        radioTime=millis();
        frame.s.battVoltADC=analogRead(A0);
        frame.s.msrLspeed=encoderLeftVel*100;
        frame.s.msrRspeed=encoderRghtVel*100;
        frame.s.rmpLspeed=speedLeftOutput*100;
        frame.s.rmpRspeed=speedRghtOutput*100;
        frame.s.msrLpwr=motorLeftPwr;
        frame.s.msrRpwr=motorRghtPwr;
        frame.s.isParked=isParked;
        calcCRC8();
        nrf24.send(frame.b, sizeof(frame.b));
        nrf24.waitPacketSent();
      }else{
        //Serial.println(F("CRC fail"));
      }
    }else{
      //Serial.println("recv fail");
    }
  }
  if(millis()-radioTime>=radioTimeout){ //set radio signal light
    PORTB &= ~(1<<PB3);
  }else{
    PORTB |= (1<<PB3);
  }
}
