byte encoderLeftCycle;          //encoder section
byte encoderLeftCycleLast;
long encoderLeftCount;
long encoderLeftCountLast;
byte encoderRghtCycle;
byte encoderRghtCycleLast;
long encoderRghtCount;
long encoderRghtCountLast;
int tdiffLeft = 0;
int tdiffRght = 0;
double encoderLeftVel=0;
double encoderRghtVel=0;
unsigned int MavgSize=100;             //How many averages to take (1 occurs every encoderVelTimestep) to filter velocity
int encoderMavgLeft[255];
int encoderMavgRght[255];
unsigned long encoderVelTimeout;
unsigned int encoderVelTimestep=1;    //how often to resolve velocity (microseconds)
#define encoderScale 21.9298245 //scales raw encoder from counts/microsecond to 10*rev/sec
int MavgCycle=0;

int ledBlinkrate=1000;

bool motorsEnabled;
int motorLeftPwr;
int motorRghtPwr;
#define     hDisablePin 18  //h-bridge  controller disable pin
#define parkingBrakePin 19  //relay that releases brake and applys power to motor (high=active)
#define   rightEnableHI 20  
#define    leftEnableHI 21
#define        rightALI 14 //PWM
#define        rightBLI 15 //PWM
#define         leftALI 12 //PWM
#define         leftBLI 13 //PWM

float speedMax=40;
float speedAccel=20;
float speedDecel=20;
float speedCutoff=0.2;
double speedLeftInput;
double speedRghtInput;
double speedLeftOutput;
double speedRghtOutput;
unsigned long speedrampTimeout;
double speedLeftPower;
double speedRghtPower;
#define speedrampTimestep 5
#include <PID_v1.h>
double speedKp=0, speedKi=70, speedKd=0; //PID Gains
PID speedLeftPID(&encoderLeftVel, &speedLeftPower, &speedLeftOutput, speedKp, speedKi, speedKd, DIRECT);
PID speedRghtPID(&encoderRghtVel, &speedRghtPower, &speedRghtOutput, speedKp, speedKi, speedKd, DIRECT);
#define speedPIDSampletime 50

byte operatingControlMode=1; //which controller to use when moving
byte controlMode=0; //which controller is currently being used (0=none, 1=PIDspeed, 2=feedfwd/power);
byte lastControlMode=0; //used for detecting when control mode changes

bool isParked=1;
#define stopTimeout 2000 //after this many millis of low enough speed and power, stop using the PID controllers, turn off and brake the motors
unsigned long autoparkTime=0;
float encoderStopVel=0.5; //motors are "stopped" when both encoder and commanded abs speeds are less than this value
int speedStopPower=6;     //motors are "stopped" when both left and right abs power are less than this value
bool radioMotorEN=0;

struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  msrLspeed;  //measured wheel speeds
  int16_t  msrRspeed;
  uint16_t battVoltADC;
  byte operatingControlMode;
  int16_t  cmdLspeed;  //speed desired
  int16_t  cmdRspeed;
  int16_t  rmpLspeed;  //speed desired, smoothed to satisfy accel/decel
  int16_t  rmpRspeed;
  uint16_t accel;      
  uint16_t decel;
  int16_t  msrLpwr;    //power the pid applies to motors
  int16_t  msrRpwr;
  bool motorEN;        //state of switch that enables driving
  bool isParked;       //brakes on or off?
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;

#include <SPI.h>
#include <RH_NRF24.h>
RH_NRF24 nrf24(1,2);
#define radioTimeout 500
unsigned long radioTime=0;

void setup(){
  pinMode(0, OUTPUT); //blue led
  pinMode(3, OUTPUT); //green led
  motorsInit();
  Serial.begin(115200);
  Serial1.begin(115200);
  if(!nrf24.init())Serial.println("init failed"); // Defaults after init are 2.402 GHz (channel 2), 2Mbps, 0dBm
  if(!nrf24.setChannel(1))Serial.println("setChannel failed");
  if(!nrf24.setRF(RH_NRF24::DataRate2Mbps, RH_NRF24::TransmitPower0dBm))Serial.println("setRF failed");    
  Serial1.print(F("Ready\r\n>"));
  encoderInit();
  speedLeftPID.SetSampleTime(speedPIDSampletime); 
  speedRghtPID.SetSampleTime(speedPIDSampletime);
  speedLeftPID.SetTunings(speedKp, speedKi, speedKd);
  speedRghtPID.SetTunings(speedKp, speedKi, speedKd);
  speedLeftPID.SetOutputLimits(-255,255);
  speedRghtPID.SetOutputLimits(-255,255);
  speedLeftPID.SetMode(AUTOMATIC);
  speedRghtPID.SetMode(AUTOMATIC);
}

void loop(){
  //serialControl();
  uUpdate();

  if(radioMotorEN==0&&frame.s.motorEN==1){radioMotorEN=1;}   //must be done outside uupdate to avoid infinate nesting
  if(radioMotorEN==1&&frame.s.motorEN==0){radioMotorEN=0; motorsDisable();}
  
  if((abs(speedLeftOutput)>speedCutoff||abs(speedRghtOutput)>speedCutoff)&&isParked==1&&radioMotorEN==1){ //if speed is called for and are stopped, unlock and enable motors and PIDs
    autoparkTime=millis();
    motorsEnable();        //unlock brakes and apply power
    controlMode=operatingControlMode;   //tell controllers to update
    isParked=0;            //status not parked
    ledBlinkrate=-1;       //blue led is constant on
  }
  
  if(isParked==0){
    if(!(abs(encoderLeftVel)<=encoderStopVel&&abs(encoderRghtVel)<=encoderStopVel&&abs(speedLeftPower)<=speedStopPower&&abs(speedRghtPower)<=speedStopPower&&abs(speedLeftOutput)<=encoderStopVel&&abs(speedRghtOutput)<=encoderStopVel)){
      autoparkTime=millis(); //stop conditions not met, so don't stop
    }
    bool noRadioSignal = (millis()-radioTime>=radioTimeout);
    if((millis()-autoparkTime>=stopTimeout)||frame.s.motorEN==0||noRadioSignal){ //we know we should execute park commands here if either stopped for a while or signal lost, or stop switch thrown
      controlMode=0;         //stop calculating pids
      motorsDisable();     //enable brakes
      isParked=1;          //update status
      ledBlinkrate=1000;    //change to blinking led
      if(noRadioSignal){ //if lost signal, will not update that motors should be stopped, so do it manually.
        radioMotorEN==0;
        frame.s.motorEN=0;
        frame.s.cmdLspeed=0;
        frame.s.cmdRspeed=0;
        speedLeftInput=0;
        speedRghtInput=0;
        speedLeftOutput=0;
        speedRghtOutput=0;
      }
    }
  }
  /*
  if(millis()%10==0){ //data stream for debugging
    //Serial.print(encoderLeftCount/456.0, 2); Serial.print('\t');
    //Serial.print(tdiffLeft); Serial.print('\t');
    Serial.print(encoderLeftVel, 2); Serial.print('\t');
    Serial.print(speedLeftInput, 2); Serial.print('\t');
    Serial.print(speedLeftOutput, 2); Serial.print('\t');
    //Serial.print(speedLeftPower, 2); Serial.print('\t');
    Serial.print(motorsEnabled); Serial.print('\t');
    Serial.print(motorLeftPwr); Serial.print('\n');
    //Serial.println();
  }*/
}

void uUpdate(){
  checkRadio();
  encoderUpdate();
  updateSpeedramps();

  if(controlMode!=0){ //if not stopped, update control mode to the new operating mode
    controlMode=operatingControlMode;
  }
  if(controlMode!=lastControlMode){ //for change of control modes
    lastControlMode=controlMode;
    switch(controlMode){
      case 1: //pids (reset them)
        speedLeftPID.SetOutputLimits(0.0, 1.0); speedLeftPID.SetOutputLimits(-1.0, 0.0); speedLeftPID.SetOutputLimits(-255,255);
        speedRghtPID.SetOutputLimits(0.0, 1.0); speedRghtPID.SetOutputLimits(-1.0, 0.0); speedRghtPID.SetOutputLimits(-255,255);
        break;
    }
  }
  if(controlMode==1){
    speedLeftPID.Compute();
    speedRghtPID.Compute();
    if(speedLeftOutput==0&&abs(encoderLeftVel)<=speedCutoff){ //turn off motor power if speed actually zero (anti winddown)
      speedLeftPID.SetOutputLimits(0.0, 1.0); speedLeftPID.SetOutputLimits(-1.0, 0.0); speedLeftPID.SetOutputLimits(-255,255);
    }
    if(speedRghtOutput==0&&abs(encoderRghtVel)<=speedCutoff){
      speedRghtPID.SetOutputLimits(0.0, 1.0); speedRghtPID.SetOutputLimits(-1.0, 0.0); speedRghtPID.SetOutputLimits(-255,255);
    }
    motorsPower(speedLeftPower, speedRghtPower);
  }
  if(controlMode==2){
    motorsPower(speedLeftOutput*10, speedRghtOutput*10);
  }
  
  if(ledBlinkrate==0){PORTB &= ~(1<<PB0);}else if(ledBlinkrate<0){PORTB |= (1<<PB0);}else{if(millis()%(2*ledBlinkrate)>ledBlinkrate){PORTB |= (1<<PB0);}else{PORTB &= ~(1<<PB0);}}
}

void uDelay(unsigned long ms){ //delay and update critical functions at the same time
  unsigned long start = millis();
  do{ uUpdate(); }while(millis() - start < ms);
}
