#include <SPI.h>
#include <RH_NRF24.h>
RH_NRF24 nrf24;

struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  rxdLspeed;
  int16_t  rxdRspeed;
  uint16_t battVolt;
  uint8_t bitStatus;
  int16_t  cmdLspeed;
  int16_t  cmdRspeed;
  uint16_t  fwdMaxSpeed;
  uint16_t  revMaxSpeed;
  uint16_t  fwdAccel;
  uint16_t  rwdDecel;
  uint16_t  revAccel;
  uint16_t  revDecel;
  uint8_t  checksum;   //CRC8 of above parameters (calculated later)
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;
byte CRC8(const byte *data, byte len) {
  //CRC-8 - based on the CRC8 formulas by Dallas/Maxim
  //code released under the therms of the GNU GPL 3.0 license
  byte crc = 0x00;
  while (len--) {
    byte extract = *data++;
    for (byte tempI = 8; tempI; tempI--) {
      byte sum = (crc ^ extract) & 0x01;
      crc >>= 1;
      if (sum) {
        crc ^= 0x8C;
      }
      extract >>= 1;
    }
  }
  return crc;
}


unsigned long cmillis=0;

void setup(){
  Serial.begin(9600);
  if(!nrf24.init())Serial.println("init failed"); // Defaults after init are 2.402 GHz (channel 2), 2Mbps, 0dBm
  if(!nrf24.setChannel(1))Serial.println("setChannel failed");
  if(!nrf24.setRF(RH_NRF24::DataRate2Mbps, RH_NRF24::TransmitPower0dBm))Serial.println("setRF failed");    
  pinMode(A3,OUTPUT);
  pinMode(A2,OUTPUT);
  Serial.println(F("WC3 Server Ready"));
  delay(1000);
}

void loop(){
  if(nrf24.available()){
    uint8_t buf[RH_NRF24_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);
    if(nrf24.recv(buf, &len)){
      //NRF24::printBuffer("request: ", buf, len);
      for(byte e=0;e<sizeof(frame.b);e++){
        frame.b[e]=buf[e];
      }
      if(checkCRC8()){
        //valid response received
        cmillis=millis();
        nrf24.send(frame.b, sizeof(frame.b));
        nrf24.waitPacketSent();
      }else{
        //Serial.println(F("CRC fail"));
      }
    }else{
      //Serial.println("recv fail");
    }
  }
  if(millis()-cmillis<500){digitalWrite(A3,HIGH);}else{digitalWrite(A3,LOW);}
}
