#include <Wire.h>
#define radioReadyPin 1 //when this pin goes high, there is a new set of valid data from radio module

#define packetsize 30 //packetsize is set within the radio module, must match it here
struct frame_t{  //structure that the control unit sends to the motor board
  int16_t  msrLspeed; //------STAT----------measured wheel speeds
  int16_t  msrRspeed; //------STAT
  uint16_t battVoltADC; //----STAT----------battery voltage
  byte operatingControlMode;//CMD
  int16_t  cmdLspeed;  //-----CMD-----------speed desired
  int16_t  cmdRspeed;  //-----CMD
  int16_t  rmpLspeed;  //-----STAT----------speed desired, smoothed to satisfy accel/decel
  int16_t  rmpRspeed;  //-----STAT
  uint16_t accel;      //-----CMD-----------accel setting of chair
  uint16_t decel;      //-----CMD
  int16_t  msrLpwr;    //-----STAT----------power the pid applies to motors
  int16_t  msrRpwr;    //-----STAT
  bool motorEN;        //-----CMD-----------state of switch that enables driving
  bool isParked;       //-----STAT----------brakes on or off?
  uint8_t  temp1;   //pad this out to 30 bytes (to match the setup of the lora-promini radios)
  uint8_t  temp2;
  uint8_t  temp3;
  uint8_t  temp4;
  uint8_t  temp5;
};
union frameU{
 frame_t s;
 uint8_t b[sizeof(frame_t)];
};
frameU frame;

int idata=0;

void setup() {
  // put your setup code here, to run once:
  Wire.begin();
  Serial.begin(115200);

  pinMode(radioReadyPin, INPUT_PULLUP);

  Serial.println(sizeof(frame.b));

}

void loop() {
  //debugFrame();
  
  if(digitalRead(radioReadyPin)==1){ //check pin to see if new valid radio message
    Wire.requestFrom(8, packetsize+1);
    byte rindex=255; //index for recived bytes
    while (Wire.available()){
      if(rindex==255){Wire.read();}else{ //don't need the first byte as this signal is now provided my radioReadyPin
        if(rindex>=packetsize-1){rindex=packetsize-1;} //overflow protection
        frame.b[rindex]=Wire.read();
      }
      rindex++;
    }
    while(digitalRead(radioReadyPin)==1){};
    //we now have valid data from the remote control loaded in frame
    idata++;
    frame.s.battVoltADC=idata;
    Wire.beginTransmission(8);
    Wire.write(frame.b, sizeof(frame.b));
    Wire.endTransmission();
  }
  
 // delay(10);

}

void debugFrame(){ //prints the Frame content to serial port
  Serial.println();
  Serial.print(F("measured speeds: "));Serial.print(frame.s.msrLspeed);Serial.print('\t');Serial.println(frame.s.msrRspeed);
  Serial.print(F("batt volt adc  : "));Serial.println(frame.s.battVoltADC);
  Serial.print(F("operControlMode: "));Serial.println(frame.s.operatingControlMode);
  Serial.print(F("comman'd speeds: "));Serial.print(frame.s.cmdLspeed);Serial.print('\t');Serial.println(frame.s.cmdRspeed);
  Serial.print(F("ramp'd   speeds: "));Serial.print(frame.s.rmpLspeed);Serial.print('\t');Serial.println(frame.s.rmpRspeed);
  Serial.print(F("accel, decel   : "));Serial.print(frame.s.accel);Serial.print('\t');Serial.println(frame.s.decel);
  Serial.print(F("motor power    : "));Serial.print(frame.s.msrLpwr);Serial.print('\t');Serial.println(frame.s.msrLpwr);
  Serial.print(F("motor enable   : "));Serial.println(frame.s.motorEN);
  Serial.print(F("isParked       : "));Serial.println(frame.s.isParked);
}
